import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:senior_platform_authentication/senior_platform_authentication.dart';

import '../../domain/entities/secure_storage_keys.dart';

abstract class SecureStorageDatasource {
  Future<void> deleteUser({required String username});
  Future<void> deleteToken({required String username});
  Future<void> deleteTokenExpirationDate({required String username});
  Future<UserModel?> readUser({required String username});
  Future<TokenModel?> readToken({required String username});
  Future<DateTime?> readTokenExpirationDate({required String username});
  Future<String?> readLastUser();
  Future<void> writeUser(UserModel user);
  Future<void> writeToken(TokenModel token);
  Future<void> writeTokenExpirationDate(
      String username, String tenantDomain, DateTime tokenExpirationDate);
  Future<void> writeLastUser(
    String username,
    String tenantDomain,
  );
  Future<void> deleteLastUser();
  Future<void> writeKey({required LoginWithKeyModel loginWithKeyModel});
  Future<void> writeKeyToken({
    required String accessKey,
    required TokenModel token,
  });
  Future<void> writeKeyTokenExpirationDate({
    required String accessKey,
    required DateTime tokenExpirationDate,
  });
  Future<void> writeLastKey({required String accessKey});
  Future<void> deleteKey({required String accessKey});
  Future<void> deleteKeyToken({required String accessKey});
  Future<void> deleteKeyTokenExpirationDate({required String accessKey});
  Future<void> deleteLastKey();
  Future<LoginWithKeyModel?> readKey({required String accessKey});
  Future<TokenModel?> readKeyToken({required String accessKey});
  Future<DateTime?> readKeyTokenExpirationDate({required String accessKey});
  Future<String?> readLastKey();
}

class SecureStorageLocalDatasource implements SecureStorageDatasource {
  late final FlutterSecureStorage _storage;

  SecureStorageLocalDatasource({FlutterSecureStorage? storage}) {
    _storage = storage ?? const FlutterSecureStorage();
  }

  @override
  Future<UserModel?> readUser({required String username}) async {
    final jsonStr =
        await _storage.read(key: '$username-${SecureStorageKeys.user}');

    if (jsonStr != null) {
      return UserModel.fromJson(jsonStr);
    }

    return null;
  }

  @override
  Future<TokenModel?> readToken({required String username}) async {
    final jsonStr =
        await _storage.read(key: '$username-${SecureStorageKeys.token}');

    if (jsonStr != null) {
      return TokenModel.fromJson(jsonStr);
    }

    return null;
  }

  @override
  Future<DateTime?> readTokenExpirationDate({required String username}) async {
    final jsonStr = await _storage.read(
        key: '$username-${SecureStorageKeys.tokenExpirationDate}');

    if (jsonStr != null) {
      return DateTime.tryParse(jsonStr);
    }

    return null;
  }

  @override
  Future<void> writeUser(UserModel user) async {
    String keyUser = user.username;
    if (!keyUser.contains('@')) {
      keyUser += '@${user.tenantDomain}';
    }
    await _storage.write(
      key: '$keyUser-${SecureStorageKeys.user}',
      value: user.toJson(),
    );
  }

  @override
  Future<void> writeToken(TokenModel token) async {
    await _storage.write(
        key: '${token.username}-${SecureStorageKeys.token}',
        value: token.toJson());
  }

  @override
  Future<void> writeTokenExpirationDate(String username, String tenantDomain,
      DateTime tokenExpirationDate) async {
    String keyUser = username;
    if (!keyUser.contains('@')) {
      keyUser += '@$tenantDomain';
    }
    await _storage.write(
        key: '$keyUser-${SecureStorageKeys.tokenExpirationDate}',
        value: tokenExpirationDate.toIso8601String());
  }

  @override
  Future<void> deleteUser({required String username}) async {
    await _storage.delete(key: '$username-${SecureStorageKeys.user}');
  }

  @override
  Future<void> deleteToken({required String username}) async {
    await _storage.delete(key: '$username-${SecureStorageKeys.token}');
  }

  @override
  Future<void> deleteTokenExpirationDate({required String username}) async {
    await _storage.delete(
        key: '$username-${SecureStorageKeys.tokenExpirationDate}');
  }

  @override
  Future<String?> readLastUser() async {
    final jsonStr = await _storage.read(key: SecureStorageKeys.lastUser);

    if (jsonStr != null) {
      return jsonStr;
    }

    return null;
  }

  @override
  Future<void> writeLastUser(String username, String tenantDomain) async {
    String keyUser = username;
    if (!keyUser.contains('@') && tenantDomain.isNotEmpty) {
      keyUser += '@$tenantDomain';
    }
    await _storage.write(key: SecureStorageKeys.lastUser, value: keyUser);
  }

  @override
  Future<void> deleteLastUser() async {
    await _storage.delete(key: SecureStorageKeys.lastUser);
  }

  @override
  Future<void> writeKey({required loginWithKeyModel}) async {
    await _storage.write(
        key: '${loginWithKeyModel.accessKey}-${SecureStorageKeys.key}',
        value: loginWithKeyModel.toJson());
  }

  @override
  Future<void> writeKeyToken({
    required String accessKey,
    required TokenModel token,
  }) async {
    await _storage.write(
        key: '$accessKey-${SecureStorageKeys.keyToken}', value: token.toJson());
  }

  @override
  Future<void> writeKeyTokenExpirationDate({
    required String accessKey,
    required DateTime tokenExpirationDate,
  }) async {
    await _storage.write(
        key: '$accessKey-${SecureStorageKeys.keyTokenExpirationDate}',
        value: tokenExpirationDate.toIso8601String());
  }

  @override
  Future<void> writeLastKey({required String accessKey}) async {
    await _storage.write(key: SecureStorageKeys.lastKey, value: accessKey);
  }

  @override
  Future<void> deleteKey({required String accessKey}) async {
    await _storage.delete(key: '$accessKey-${SecureStorageKeys.key}');
  }

  @override
  Future<void> deleteKeyToken({required String accessKey}) async {
    await _storage.delete(key: '$accessKey-${SecureStorageKeys.keyToken}');
  }

  @override
  Future<void> deleteKeyTokenExpirationDate({required String accessKey}) async {
    await _storage.delete(
        key: '$accessKey-${SecureStorageKeys.keyTokenExpirationDate}');
  }

  @override
  Future<void> deleteLastKey() async {
    await _storage.delete(key: SecureStorageKeys.lastKey);
  }

  @override
  Future<LoginWithKeyModel?> readKey({required String accessKey}) async {
    final jsonStr =
        await _storage.read(key: '$accessKey-${SecureStorageKeys.key}');

    if (jsonStr != null) {
      return LoginWithKeyModel.fromJson(jsonStr);
    }

    return null;
  }

  @override
  Future<TokenModel?> readKeyToken({required String accessKey}) async {
    final jsonStr =
        await _storage.read(key: '$accessKey-${SecureStorageKeys.keyToken}');

    if (jsonStr != null) {
      return TokenModel.fromJson(jsonStr);
    }

    return null;
  }

  @override
  Future<DateTime?> readKeyTokenExpirationDate(
      {required String accessKey}) async {
    final jsonStr = await _storage.read(
        key: '$accessKey-${SecureStorageKeys.keyTokenExpirationDate}');

    if (jsonStr != null) {
      return DateTime.tryParse(jsonStr);
    }

    return null;
  }

  @override
  Future<String?> readLastKey() async {
    final jsonStr = await _storage.read(key: SecureStorageKeys.lastKey);

    if (jsonStr != null) {
      return jsonStr;
    }

    return null;
  }
}
