import 'dart:async';
import 'dart:developer';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import '../../domain/entities/key_authentication_data.dart';

import '../../../../../senior_platform_authentication_ui.dart';

part 'authentication_event.dart';
part 'authentication_state.dart';

/// Authentication business logic component.
///
/// Manages authentication status based on initialize configuration.
/// This instance must be a singleton and must be initialized closest to
/// the root of your application's widget tree.
class AuthenticationBloc
    extends Bloc<AuthenticationEvent, AuthenticationState> {
  final GetUserUsecase _getUserUsecase;
  final StoreAuthenticationDataUsecase _storeAuthenticationDataUsecase;
  final CheckStoredTokenUsecase _checkStoredTokenUsecase;
  final LogoutUsecase _logoutUsecase;
  final ClearStoredDataUsecase _clearStoredDataUsecase;
  final GetStoredTokenUsecase _getStoredTokenUsecase;
  final RefreshStoredTokenUsecase _refreshStoredTokenUsecase;
  final CheckStatusConnectionUsecase _checkStatusConnectionUsecase;
  final StoreKeyAuthenticationDataUsecase _storeKeyAuthenticationDataUsecase;
  final CheckKeyStoredTokenUsecase _checkKeyStoredTokenUsecase;
  final RefreshKeyStoredTokenUsecase _refreshKeyStoredTokenUsecase;
  final GetStoredKeyTokenUsecase _getStoredKeyTokenUsecase;
  final ClearKeyStoredDataUsecase _clearKeyStoredDataUsecase;
  final GetStoredKeyUsecase _getStoredKeyUsecase;
  final AuthenticateKeyUsecase _authenticateKeyUsecase;
  final BiometricAuthenticateUsecase _biometricAuthAuthenticateUsecase;
  final BiometricCanCheckUseCase _biometricAuthCanAuthenticateUseCase;
  final GetStoredUserUsecase _getStoredUserUsecase;

  /// Internal constructor, should not be exposed.
  AuthenticationBloc._internal({
    required GetUserUsecase getUserUsecase,
    required StoreAuthenticationDataUsecase storeAuthenticationDataUsecase,
    required CheckStoredTokenUsecase checkStoredTokenUsecase,
    required LogoutUsecase logoutUsecase,
    required ClearStoredDataUsecase clearStoredDataUsecase,
    required GetStoredTokenUsecase getStoredTokenUsecase,
    required RefreshStoredTokenUsecase refreshStoredTokenUsecase,
    required CheckStatusConnectionUsecase checkStatusConnectionUsecase,
    required StoreKeyAuthenticationDataUsecase
        storeAppKeyAuthenticationDataUsecase,
    required CheckKeyStoredTokenUsecase checkKeyStoredTokenUsecase,
    required RefreshKeyStoredTokenUsecase refreshKeyStoredTokenUsecase,
    required GetStoredKeyTokenUsecase getStoredKeyTokenUsecase,
    required ClearKeyStoredDataUsecase clearKeyStoredDataUsecase,
    required GetStoredKeyUsecase getStoredKeyUsecase,
    required AuthenticateKeyUsecase authenticateKeyUsecase,
    required BiometricAuthenticateUsecase biometricAuthAuthenticateUsecase,
    required BiometricCanCheckUseCase biometricAuthCanAuthenticateUsecase,
    required GetStoredUserUsecase getStoredUserUsecase,
  })  : _getUserUsecase = getUserUsecase,
        _storeAuthenticationDataUsecase = storeAuthenticationDataUsecase,
        _checkStoredTokenUsecase = checkStoredTokenUsecase,
        _logoutUsecase = logoutUsecase,
        _clearStoredDataUsecase = clearStoredDataUsecase,
        _getStoredTokenUsecase = getStoredTokenUsecase,
        _refreshStoredTokenUsecase = refreshStoredTokenUsecase,
        _checkStatusConnectionUsecase = checkStatusConnectionUsecase,
        _storeKeyAuthenticationDataUsecase =
            storeAppKeyAuthenticationDataUsecase,
        _checkKeyStoredTokenUsecase = checkKeyStoredTokenUsecase,
        _refreshKeyStoredTokenUsecase = refreshKeyStoredTokenUsecase,
        _getStoredKeyTokenUsecase = getStoredKeyTokenUsecase,
        _clearKeyStoredDataUsecase = clearKeyStoredDataUsecase,
        _getStoredKeyUsecase = getStoredKeyUsecase,
        _authenticateKeyUsecase = authenticateKeyUsecase,
        _biometricAuthAuthenticateUsecase = biometricAuthAuthenticateUsecase,
        _biometricAuthCanAuthenticateUseCase =
            biometricAuthCanAuthenticateUsecase,
        _getStoredUserUsecase = getStoredUserUsecase,
        super(const AuthenticationState.unknown()) {
    on<AuthenticateKey>(_onAuthenticateKey);
    on<AuthenticationStatusChanged>(_onAuthenticationStatusChanged);
    on<LogoutOnlineRequested>(_onLogoutOnlineRequested);
    on<CheckAuthenticationRequested>(_onCheckAuthenticationRequested);
    on<LogoutOfflineRequested>(_onLogoutOfflineRequested);
    on<ErrorAuthentication>(_onErrorAuthentication);
  }

  /// Singleton instance for production environment.
  static final AuthenticationBloc _instance = AuthenticationBloc._internal(
    getUserUsecase: GetUserUsecase(),
    storeAuthenticationDataUsecase: StoreAuthenticationDataUsecase(),
    checkStoredTokenUsecase: CheckStoredTokenUsecase(
      getUserUsecase: GetUserUsecase(),
      checkStatusConnectionUsecase: CheckStatusConnectionUsecase(
          getConnectivityStatusUsecase: GetConnectivityStatusUsecase()),
    ),
    logoutUsecase: LogoutUsecase(),
    clearStoredDataUsecase: ClearStoredDataUsecase(),
    getStoredTokenUsecase: GetStoredTokenUsecase(),
    refreshStoredTokenUsecase: RefreshStoredTokenUsecase(
      getUserUsecase: GetUserUsecase(),
      refreshTokenUsecase: RefreshTokenUsecase(),
      storeAuthenticationDataUsecase: StoreAuthenticationDataUsecase(),
      clearStoredDataUsecase: ClearStoredDataUsecase(),
    ),
    checkStatusConnectionUsecase: CheckStatusConnectionUsecase(
        getConnectivityStatusUsecase: GetConnectivityStatusUsecase()),
    storeAppKeyAuthenticationDataUsecase: StoreKeyAuthenticationDataUsecase(),
    checkKeyStoredTokenUsecase: CheckKeyStoredTokenUsecase(),
    refreshKeyStoredTokenUsecase: RefreshKeyStoredTokenUsecase(),
    getStoredKeyTokenUsecase: GetStoredKeyTokenUsecase(),
    clearKeyStoredDataUsecase: ClearKeyStoredDataUsecase(),
    getStoredKeyUsecase: GetStoredKeyUsecase(),
    authenticateKeyUsecase: AuthenticateKeyUsecase(),
    biometricAuthAuthenticateUsecase: BiometricAuthenticateUsecase(),
    biometricAuthCanAuthenticateUsecase: BiometricCanCheckUseCase(),
    getStoredUserUsecase: GetStoredUserUsecase(),
  );

  /// Factory exposed constructor.
  factory AuthenticationBloc() => _instance;

  /// Factory mocked exposed constructor. Should only be used for tests.
  factory AuthenticationBloc.mocked({
    required GetUserUsecase getUserUsecase,
    required StoreAuthenticationDataUsecase storeAuthenticationDataUsecase,
    required CheckStoredTokenUsecase checkStoredTokenUsecase,
    required LogoutUsecase logoutOnlineUsecase,
    required ClearStoredDataUsecase clearStoredDataUsecase,
    required GetStoredTokenUsecase getStoredTokenUsecase,
    required RefreshStoredTokenUsecase refreshStoredTokenUsecase,
    required CheckStatusConnectionUsecase checkStatusConnectionUsecase,
    required StoreKeyAuthenticationDataUsecase
        storeAppKeyAuthenticationDataUsecase,
    required CheckKeyStoredTokenUsecase checkKeyStoredTokenUsecase,
    required RefreshKeyStoredTokenUsecase refreshKeyStoredTokenUsecase,
    required GetStoredKeyTokenUsecase getStoredKeyTokenUsecase,
    required ClearKeyStoredDataUsecase clearKeyStoredDataUsecase,
    required GetStoredKeyUsecase getStoredKeyUsecase,
    required AuthenticateKeyUsecase authenticateKeyUsecase,
    required BiometricAuthenticateUsecase biometricAuthAuthenticateUseCase,
    required BiometricCanCheckUseCase biometricAuthCanAuthenticateUseCase,
    required GetStoredUserUsecase getStoredUserUsecase,
  }) {
    return AuthenticationBloc._internal(
      getUserUsecase: getUserUsecase,
      storeAuthenticationDataUsecase: storeAuthenticationDataUsecase,
      checkStoredTokenUsecase: checkStoredTokenUsecase,
      logoutUsecase: logoutOnlineUsecase,
      clearStoredDataUsecase: clearStoredDataUsecase,
      getStoredTokenUsecase: getStoredTokenUsecase,
      refreshStoredTokenUsecase: refreshStoredTokenUsecase,
      checkStatusConnectionUsecase: checkStatusConnectionUsecase,
      storeAppKeyAuthenticationDataUsecase:
          storeAppKeyAuthenticationDataUsecase,
      checkKeyStoredTokenUsecase: checkKeyStoredTokenUsecase,
      refreshKeyStoredTokenUsecase: refreshKeyStoredTokenUsecase,
      getStoredKeyTokenUsecase: getStoredKeyTokenUsecase,
      clearKeyStoredDataUsecase: clearKeyStoredDataUsecase,
      getStoredKeyUsecase: getStoredKeyUsecase,
      authenticateKeyUsecase: authenticateKeyUsecase,
      biometricAuthAuthenticateUsecase: biometricAuthAuthenticateUseCase,
      biometricAuthCanAuthenticateUsecase: biometricAuthCanAuthenticateUseCase,
      getStoredUserUsecase: getStoredUserUsecase,
    );
  }

  // Authenticate pre-configured keys
  Future<void> _onAuthenticateKey(
    AuthenticateKey event,
    Emitter<AuthenticationState> emit,
  ) async {
    if (SeniorAuthentication.enableLoginWithKey) {
      final result = await _authenticateKeyUsecase.call(event.accessKey);

      if (result == null) {
        return emit(const AuthenticationState.keyUnauthenticated());
      }

      add(
        AuthenticationStatusChanged(
          AuthenticationStatus.authenticated,
          authenticationResponse: result,
        ),
      );
    } else {
      return emit(const AuthenticationState.unknown());
    }
  }

  Future<void> _onAuthenticationStatusChanged(
    AuthenticationStatusChanged event,
    Emitter<AuthenticationState> emit,
  ) async {
    SeniorAuthentication.enableLoginWithKey
        ? await _onKeyAuthenticationStatusChanged(event, emit)
        : await _onUserAuthenticationStatusChanged(event, emit);
  }

  Future<void> _onKeyAuthenticationStatusChanged(
    AuthenticationStatusChanged event,
    Emitter<AuthenticationState> emit,
  ) async {
    switch (event.status) {
      case AuthenticationStatus.unauthenticated:
        return emit(const AuthenticationState.keyUnauthenticated());
      case AuthenticationStatus.authenticated:
        final stored = await _storeKeyAuthenticationDataUsecase(
          KeyAuthenticationData(
            loginWithKey: event.authenticationResponse?.loginWithKey,
            token: event.authenticationResponse?.token,
          ),
        );
        return emit(stored
            ? AuthenticationState.keyAuthenticated(
                accessKey:
                    event.authenticationResponse?.loginWithKey?.accessKey)
            : const AuthenticationState.keyUnauthenticated());
      case AuthenticationStatus.unknown:
      case AuthenticationStatus.offline:
        return emit(const AuthenticationState.unknown());
    }
  }

  Future<void> _onUserAuthenticationStatusChanged(
    AuthenticationStatusChanged event,
    Emitter<AuthenticationState> emit,
  ) async {
    switch (event.status) {
      case AuthenticationStatus.unauthenticated:
        return emit(const AuthenticationState.unauthenticated());
      case AuthenticationStatus.authenticated:
        final user = await _tryGetUser(
          GetUserInput(
            accessToken: event.authenticationResponse?.token?.accessToken ?? '',
            includePhoto: SeniorAuthentication.includePhoto,
          ),
          biometricEnabled: event.biometryStatus == BiometryStatus.success,
        );

        final stored = await _storeAuthenticationDataUsecase(
          AuthenticationData(
            user: user,
            token: event.authenticationResponse?.token,
          ),
        );

        final bool biometricEnabled =
            event.biometryStatus == BiometryStatus.success;
        if (biometricEnabled && stored) {
          if (biometricEnabled) {
            return emit(AuthenticationState.authenticated(
                username: event.authenticationResponse?.token?.username));
          } else {
            const username = UserName();
            await _clearStoredDataUsecase(username);
            return emit(
              AuthenticationState.unauthenticated(
                biometryStatus: event.biometryStatus,
              ),
            );
          }
        }

        return emit(stored
            ? AuthenticationState.authenticated(
                username: event.authenticationResponse?.token?.username)
            : const AuthenticationState.unauthenticated());
      case AuthenticationStatus.unknown:
        return emit(const AuthenticationState.unknown());
      case AuthenticationStatus.offline:
        return emit(AuthenticationState.offline(
            event.authenticationResponse?.token?.username ?? ''));
    }
  }

  void _onLogoutOnlineRequested(
    LogoutOnlineRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    SeniorAuthentication.enableLoginWithKey
        ? await _onKeyLogoutOnlineRequested(event, emit)
        : await _onUserLogoutOnlineRequested(event, emit);
  }

  Future<void> _onKeyLogoutOnlineRequested(
    LogoutOnlineRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    ApplicationKey? key = await _getStoredKeyUsecase(null);

    try {
      final isOnline = await _checkStatusConnectionUsecase(NoParams());

      if (isOnline) {
        final token = await _getStoredKeyTokenUsecase(key!.accessKey);
        if (token != null) {
          await _logoutUsecase(token.accessToken);
        }
      }
    } catch (err) {
      log(err.toString());
    } finally {
      await _clearKeyStoredDataUsecase(key!.accessKey);
      AuthenticationState newState =
          const AuthenticationState.keyUnauthenticated();
      emit(newState);
    }
  }

  Future<void> _onUserLogoutOnlineRequested(
    LogoutOnlineRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    const username = UserName();
    try {
      final isOnline = await _checkStatusConnectionUsecase(NoParams());

      if (isOnline) {
        final token = await _getStoredTokenUsecase(username);
        if (token != null) {
          await _logoutUsecase(token.accessToken);
        }
      }
    } catch (err) {
      log(err.toString());
    } finally {
      await _clearStoredDataUsecase(username);
      emit(const AuthenticationState.unauthenticated());
    }
  }

  void _onLogoutOfflineRequested(
    LogoutOfflineRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    final username = UserName(currentUsername: event.username);
    final token = await _getStoredTokenUsecase(username);
    if (event.eraseToken && token != null) {
      await _clearStoredDataUsecase(username);
    } else {
      //delete only last user stored
      await _clearStoredDataUsecase(const UserName(currentUsername: ''));
    }

    emit(const AuthenticationState.unauthenticated());
  }

  Future<void> _onCheckAuthenticationRequested(
    CheckAuthenticationRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    SeniorAuthentication.enableLoginWithKey
        ? await _onCheckKeyAuthenticationRequested(event, emit)
        : await _onCheckUserAuthenticationRequested(event, emit);
  }

  Future<void> _onCheckKeyAuthenticationRequested(
    CheckAuthenticationRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    final isTokenValid = await _checkKeyStoredTokenUsecase(event.accesskey);

    if (!isTokenValid) {
      final token = await _refreshKeyStoredTokenUsecase.call(event.accesskey);

      return emit(token != null
          ? AuthenticationState.keyAuthenticated(accessKey: event.accesskey)
          : const AuthenticationState.keyUnauthenticated());
    }
    emit(AuthenticationState.keyAuthenticated(accessKey: event.accesskey));
  }

  Future<void> _onCheckUserAuthenticationRequested(
    CheckAuthenticationRequested event,
    Emitter<AuthenticationState> emit,
  ) async {
    final username = UserName(currentUsername: event.username);
    final isTokenValid = await _checkStoredTokenUsecase(username);
    final storedUser = await _getStoredUserUsecase(username);
    BiometryStatus biometryStatus = BiometryStatus.unknown;

    if (!isTokenValid) {
      final token = await _refreshStoredTokenUsecase(username);
      return emit(token != null
          ? AuthenticationState.authenticated(username: event.username)
          : const AuthenticationState.unauthenticated());
    }

    if (storedUser != null) {
      if (SeniorAuthentication.enableBiometry && storedUser.activeBiometry) {
        biometryStatus = await onAuthenticationBiometricAuthRequested(
            biometricMessage: event.biometricMessage ?? '');

        if (biometryStatus == BiometryStatus.canceled ||
            biometryStatus == BiometryStatus.error) {
          return emit(AuthenticationState.unauthenticated(
              biometryStatus: biometryStatus));
        }
      }
    }

    return emit(
      AuthenticationState.authenticated(
        username: event.username,
        biometryStatus: biometryStatus,
      ),
    );
  }

  Future<User?> _tryGetUser(GetUserInput getUserInput,
      {required bool biometricEnabled}) async {
    try {
      User user = await _getUserUsecase(getUserInput);
      user = user.copyWith(
        activeBiometry: biometricEnabled,
      );
      return user;
    } catch (_) {
      return null;
    }
  }

  Future<void> _onErrorAuthentication(
    ErrorAuthentication event,
    Emitter<AuthenticationState> emit,
  ) async {
    const username = UserName();
    await _clearStoredDataUsecase(username);
    return emit(
      const AuthenticationState.unauthenticated(),
    );
  }

  // Method that calls biometrics
  Future<BiometryStatus> onAuthenticationBiometricAuthRequested(
      {required String biometricMessage}) async {
    try {
      BiometryStatus biometryStatus = BiometryStatus.unknown;
      final canAuthenticate =
          await _biometricAuthCanAuthenticateUseCase(NoParams());
      if (canAuthenticate) {
        biometryStatus = await _biometricAuthAuthenticateUsecase(
          biometricMessage,
        );
      } else {
        biometryStatus = BiometryStatus.canceled;
      }
      return biometryStatus;
    } catch (e) {
      return BiometryStatus.error;
    }
  }
}
