part of 'authentication_bloc.dart';

/// Base authentication event class.
abstract class AuthenticationEvent extends Equatable {
  const AuthenticationEvent();
}

class AuthenticateKey extends AuthenticationEvent {
  final String accessKey;

  const AuthenticateKey(this.accessKey);

  @override
  List<Object?> get props => [
        accessKey,
      ];
}

class AuthenticationBiometricAuthRequested extends AuthenticationEvent {
  /// Flag that indicates if biometric authentication is enabled.
  final bool? biometricEnabled;
  // variable responsible for the message that appears in the biometrics dialog
  final String biometricMessage;

  final AuthenticationStatus status;

  const AuthenticationBiometricAuthRequested({
    this.biometricEnabled = false,
    this.biometricMessage = '',
    this.status = AuthenticationStatus.authenticated,
  });

  @override
  List<Object?> get props => [
        biometricEnabled,
        biometricMessage,
        status,
      ];
}

/// Fired everytime user's authentication status changes.
class AuthenticationStatusChanged extends AuthenticationEvent {
  /// Current user's authentication status.
  final AuthenticationStatus status;

  /// Authentication response must be passed when the status is
  /// [AuthenticationStatus.authenticated].
  final AuthenticationResponse? authenticationResponse;

  final BiometryStatus biometryStatus;

  const AuthenticationStatusChanged(
    this.status, {
    this.authenticationResponse,
    this.biometryStatus = BiometryStatus.unknown,
  });

  @override
  List<Object?> get props => [
        status,
        authenticationResponse,
        biometryStatus,
      ];
}

/// Calls logout when login offline is disabled.
class LogoutOnlineRequested extends AuthenticationEvent {
  const LogoutOnlineRequested();

  @override
  List<Object?> get props => [];
}

/// Calls logout when login offline is enabled.
class LogoutOfflineRequested extends AuthenticationEvent {
  /// Current username saved on storage.
  final String username;

  /// Flag that erase token from sotrage.
  final bool eraseToken;

  const LogoutOfflineRequested({
    required this.username,
    this.eraseToken = false,
  });

  @override
  List<Object?> get props => [
        username,
        eraseToken,
      ];
}

/// Check if the user has a valid authentication.
///
/// You must call this event as soon as you instantiate the authentication bloc
/// for the provider. See the documentation with examples for more details.
class CheckAuthenticationRequested extends AuthenticationEvent {
  /// Current username saved on storage.
  final String? username;
  final String? accesskey;
  final String? biometricMessage;

  const CheckAuthenticationRequested({
    this.username,
    this.accesskey,
    this.biometricMessage,
  });

  @override
  List<Object?> get props => [
        username,
        accesskey,
        biometricMessage,
      ];
}

class CheckBiometricAuthenticationRequested extends AuthenticationEvent {
  /// Current username saved on storage.
  final String biometricMessage;

  const CheckBiometricAuthenticationRequested({
    required this.biometricMessage,
  });

  @override
  List<Object?> get props => [
        biometricMessage,
      ];
}

class ErrorAuthentication extends AuthenticationEvent {
  const ErrorAuthentication();

  @override
  List<Object?> get props => [];
}
