import 'package:bloc_test/bloc_test.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:senior_platform_authentication_ui/senior_platform_authentication_ui.dart';
import 'package:senior_platform_authentication_ui/src/presentation/mfa_authentication/cubit/mfa_authentication_code_cubit.dart';

import '../../../../mocks/mfa_info_mock.dart';
import '../../../../mocks/token_mock.dart';

class MockLoginMfaUseCase extends Mock implements LoginMFAUsecase {}

class MockAuthenticationBloc extends Mock implements AuthenticationBloc {}

class MockSendMfaConfigEmailUsecase extends Mock
    implements SendMFAConfigEmailUsecase {}

void main() {
  late LoginMFAUsecase loginMfaUseCase;
  late AuthenticationBloc authenticationBloc;
  late SendMFAConfigEmailUsecase sendMfaConfigEmailUsecase;
  late MFAAuthenticationCubit mfaAuthenticationCubit;

  setUpAll(
    () => SeniorAuthentication.initialize(),
  );

  setUp(() {
    loginMfaUseCase = MockLoginMfaUseCase();
    authenticationBloc = MockAuthenticationBloc();
    sendMfaConfigEmailUsecase = MockSendMfaConfigEmailUsecase();
    mfaAuthenticationCubit = MFAAuthenticationCubit(
      loginMFAUsecase: loginMfaUseCase,
      authenticationBloc: authenticationBloc,
      sendMfaConfigEmailUsecase: sendMfaConfigEmailUsecase,
    );
  });

  group(
    'initialize',
    () {
      test('should emit [MFAAuthenticationState.initial()] when initialize',
          () {
        expect(mfaAuthenticationCubit.state, MFAAuthenticationState.initial());
      });
    },
  );

  group('initialize enableBiometry', () {
    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
        'emits correct states when login executes successfully',
        setUp: () => when(
              () => loginMfaUseCase.call(
                loginMFAMock,
              ),
            ).thenAnswer(
              (_) async => authenticationResponseMock,
            ),
        build: () => mfaAuthenticationCubit,
        act: (cubit) => cubit.login(
              validationCode: '112233',
              temporaryToken:
                  'Dr9BFR82r1W7VKbEikNfmJ3Ch0a+U/YXToC/OdkriM6R2LyRxrLyvBz5aypBMVfqd3ek7Jjz5EdCbdHYo5p2g2GJalzFFvVkNxiLBfIBTseYZa69wyZLxA==',
              tenant: 'apphcmcom',
            ),
        expect: () => <MFAAuthenticationState>[
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.loading,
                  errorType: null),
            ]);
  });

  group('initialize', () {
    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
      'description',
      build: () => MFAAuthenticationCubit(
        loginMFAUsecase: loginMfaUseCase,
        authenticationBloc: authenticationBloc,
        sendMfaConfigEmailUsecase: sendMfaConfigEmailUsecase,
      ),
      act: (cubit) => cubit.initialize(mfaInfoConfiguredMock),
      expect: () => [
        MFAAuthenticationState.initial().copyWith(
          mfaStatus: MFAAuthenticationStatus.configured,
          mfaInfo: mfaInfoConfiguredMock,
        ),
      ],
    );

    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
        'emits correct states when login executes successfully',
        setUp: () => when(
              () => loginMfaUseCase.call(
                loginMFAMock,
              ),
            ).thenAnswer(
              (_) async => authenticationResponseMock,
            ),
        build: () => mfaAuthenticationCubit,
        act: (cubit) => cubit.login(
              validationCode: '112233',
              temporaryToken:
                  'Dr9BFR82r1W7VKbEikNfmJ3Ch0a+U/YXToC/OdkriM6R2LyRxrLyvBz5aypBMVfqd3ek7Jjz5EdCbdHYo5p2g2GJalzFFvVkNxiLBfIBTseYZa69wyZLxA==',
              tenant: 'apphcmcom',
            ),
        expect: () => <MFAAuthenticationState>[
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.loading,
                  errorType: null),
            ]);

    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
        'emits correct states when loginMfa fails with UnauthorizedException',
        setUp: () => when(
              () => loginMfaUseCase.call(
                loginMFAMock,
              ),
            ).thenThrow(
              UnauthorizedException(),
            ),
        build: () => mfaAuthenticationCubit,
        act: (cubit) async => await cubit.login(
              validationCode: '112233',
              temporaryToken:
                  'Dr9BFR82r1W7VKbEikNfmJ3Ch0a+U/YXToC/OdkriM6R2LyRxrLyvBz5aypBMVfqd3ek7Jjz5EdCbdHYo5p2g2GJalzFFvVkNxiLBfIBTseYZa69wyZLxA==',
              tenant: 'apphcmcom',
            ),
        expect: () => <MFAAuthenticationState>[
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.loading,
                  errorType: null),
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.idle,
                  errorType: ErrorType.unauthorized),
            ]);

    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
        'emits correct states when loginMfa fails with Unknown error',
        setUp: () => when(
              () => loginMfaUseCase.call(
                loginMFAMock,
              ),
            ).thenThrow(
              Exception,
            ),
        build: () => mfaAuthenticationCubit,
        act: (cubit) async => await cubit.login(
              validationCode: '112233',
              temporaryToken:
                  'Dr9BFR82r1W7VKbEikNfmJ3Ch0a+U/YXToC/OdkriM6R2LyRxrLyvBz5aypBMVfqd3ek7Jjz5EdCbdHYo5p2g2GJalzFFvVkNxiLBfIBTseYZa69wyZLxA==',
              tenant: 'apphcmcom',
            ),
        expect: () => <MFAAuthenticationState>[
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.loading,
                  errorType: null),
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.idle,
                  errorType: ErrorType.unknown),
            ]);

    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
        'emits correct states when sendMFAConfigEmail executes successfully',
        setUp: () => when(
              () => sendMfaConfigEmailUsecase.call(
                sendMFAConfigEmailMock,
              ),
            ).thenAnswer(
              (_) async => const EmailAddress(
                email: 'teste@senior.com.br',
              ),
            ),
        build: () => mfaAuthenticationCubit,
        act: (cubit) => cubit.sendMFAConfigEmail(
              temporaryToken: 'temporaryToken',
              tenant: 'tenant',
            ),
        expect: () => <MFAAuthenticationState>[
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.loading,
                  errorType: null),
              const MFAAuthenticationState(
                  email: 'teste@senior.com.br',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.emailSended,
                  status: NetworkStatus.idle,
                  errorType: null),
            ]);

    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
        'emits correct states when sendMFAConfigEmail fails with UnauthorizedException',
        setUp: () => when(
              () => sendMfaConfigEmailUsecase.call(
                sendMFAConfigEmailMock,
              ),
            ).thenThrow(
              UnauthorizedException(),
            ),
        build: () => mfaAuthenticationCubit,
        act: (cubit) => cubit.sendMFAConfigEmail(
              temporaryToken:
                  'Dr9BFR82r1W7VKbEikNfmJ3Ch0a+U/YXToC/OdkriM6R2LyRxrLyvBz5aypBMVfqd3ek7Jjz5EdCbdHYo5p2g2GJalzFFvVkNxiLBfIBTseYZa69wyZLxA==',
              tenant: 'apphcmcom',
            ),
        expect: () => <MFAAuthenticationState>[
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.loading,
                  errorType: null),
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.idle,
                  errorType: ErrorType.unknown),
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.idle,
                  errorType: null),
            ]);

    blocTest<MFAAuthenticationCubit, MFAAuthenticationState>(
        'emits correct states when sendMFAConfigEmail fails with Unknown Error',
        setUp: () => when(
              () => sendMfaConfigEmailUsecase.call(
                sendMFAConfigEmailMock,
              ),
            ).thenThrow(
              Exception(),
            ),
        build: () => mfaAuthenticationCubit,
        act: (cubit) => cubit.sendMFAConfigEmail(
              temporaryToken:
                  'Dr9BFR82r1W7VKbEikNfmJ3Ch0a+U/YXToC/OdkriM6R2LyRxrLyvBz5aypBMVfqd3ek7Jjz5EdCbdHYo5p2g2GJalzFFvVkNxiLBfIBTseYZa69wyZLxA==',
              tenant: 'apphcmcom',
            ),
        expect: () => <MFAAuthenticationState>[
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.loading,
                  errorType: null),
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.idle,
                  errorType: ErrorType.unknown),
              const MFAAuthenticationState(
                  email: '',
                  mfaInfo: MFAInfo(),
                  mfaStatus: MFAAuthenticationStatus.unknown,
                  status: NetworkStatus.idle,
                  errorType: null),
            ]);
  });
}
